@@ -1,26 +1,38 @@
-from __future__ import annotations
-
 import ast
-import asyncio
+import threading
+from concurrent.futures import CancelledError
 from typing import TYPE_CHECKING, Any, List, Optional
 
-from robotcode.core.async_tools import check_canceled, create_sub_task, threaded
-from robotcode.core.logging import LoggingDescriptor
-from robotcode.core.lsp.types import Diagnostic, DiagnosticSeverity, DiagnosticTag, Position, Range
-from robotcode.core.uri import Uri
+from robot.parsing.lexer.tokens import Token
 
-from ...common.decorators import language_id
-from ...common.parts.diagnostics import DiagnosticsResult
-from ...common.text_document import TextDocument
-from ..configuration import AnalysisConfig
-from ..diagnostics.entities import ArgumentDefinition
-from ..diagnostics.namespace import Namespace
-from ..utils.ast_utils import (
-    HeaderAndBodyBlock,
-    Token,
+from robotcode.core.concurrent import check_current_task_canceled
+from robotcode.core.language import language_id
+from robotcode.core.lsp.types import (
+    Diagnostic,
+    DiagnosticSeverity,
+    DiagnosticTag,
+    Position,
+    Range,
+)
+from robotcode.core.text_document import TextDocument
+from robotcode.core.uri import Uri
+from robotcode.core.utils.logging import LoggingDescriptor
+from robotcode.language_server.robotframework.configuration import AnalysisConfig
+from robotcode.robot.diagnostics.entities import (
+    ArgumentDefinition,
+    CommandLineVariableDefinition,
+    EnvironmentVariableDefinition,
+    LibraryArgumentDefinition,
+)
+from robotcode.robot.diagnostics.namespace import Namespace
+from robotcode.robot.utils.ast import (
+    iter_nodes,
     range_from_node,
     range_from_token,
 )
+from robotcode.robot.utils.stubs import HasError, HasErrors, HeaderAndBodyBlock
+
+from ...common.parts.diagnostics import DiagnosticsResult
 
 if TYPE_CHECKING:
     from ..protocol import RobotLanguageServerProtocol
@@ -31,56 +43,77 @@ from .protocol_part import RobotLanguageServerProtocolPart
 class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
     _logger = LoggingDescriptor()
 
-    def __init__(self, parent: RobotLanguageServerProtocol) -> None:
+    def __init__(self, parent: "RobotLanguageServerProtocol") -> None:
         super().__init__(parent)
 
         self.source_name = "robotcode.diagnostics"
 
-        parent.diagnostics.collect.add(self.collect_token_errors)
-        parent.diagnostics.collect.add(self.collect_model_errors)
+        self.parent.on_initialized.add(self._on_initialized)
 
-        parent.diagnostics.collect.add(self.collect_namespace_diagnostics)
+        self.parent.diagnostics.collect.add(self.collect_token_errors)
+        self.parent.diagnostics.collect.add(self.collect_model_errors)
 
-        parent.diagnostics.collect.add(self.collect_unused_keyword_references)
-        parent.diagnostics.collect.add(self.collect_unused_variable_references)
+        self.parent.diagnostics.collect.add(self.collect_namespace_diagnostics)
 
-        parent.documents_cache.namespace_invalidated.add(self.namespace_invalidated)
+        self.parent.diagnostics.collect.add(self.collect_unused_keyword_references)
+        self.parent.diagnostics.collect.add(self.collect_unused_variable_references)
 
-    async def namespace_invalidated_task(self, namespace: Namespace) -> None:
+        self._collect_unused_references_event = threading.Event()
+
+        self.parent.diagnostics.on_workspace_diagnostics_analyze.add(self._on_workspace_diagnostics_analyze)
+        self.parent.diagnostics.on_workspace_diagnostics_collect.add(self._on_workspace_diagnostics_collect)
+
+    def _on_initialized(self, sender: Any) -> None:
+        self.parent.diagnostics.analyze.add(self.analyze)
+
+        self.parent.documents_cache.namespace_invalidated.add(self.namespace_invalidated)
+
+    def _on_workspace_diagnostics_analyze(self, sender: Any) -> None:
+        self._collect_unused_references_event.clear()
+
+    def _on_workspace_diagnostics_collect(self, sender: Any) -> None:
+        self._collect_unused_references_event.set()
+
+    @language_id("robotframework")
+    def analyze(self, sender: Any, document: TextDocument) -> None:
+        self.parent.documents_cache.get_namespace(document).analyze()
+
+    @language_id("robotframework")
+    def namespace_invalidated(self, sender: Any, namespace: Namespace) -> None:
+        self._collect_unused_references_event.clear()
+
+        self._namespace_invalidated(namespace)
+
+        self.parent.diagnostics.break_workspace_diagnostics_loop()
+
+    def _namespace_invalidated(self, namespace: Namespace) -> None:
         if namespace.document is not None:
             refresh = namespace.document.opened_in_editor
 
-            await self.parent.diagnostics.force_refresh_document(namespace.document, False)
+            self.parent.diagnostics.force_refresh_document(namespace.document, False)
 
-            if await namespace.is_initialized():
-                resources = (await namespace.get_resources()).values()
+            if namespace.is_initialized():
+                resources = namespace.get_resources().values()
                 for r in resources:
                     if r.library_doc.source:
                         doc = self.parent.documents.get(Uri.from_path(r.library_doc.source).normalized())
                         if doc is not None:
                             refresh |= doc.opened_in_editor
-                            await self.parent.diagnostics.force_refresh_document(doc, False)
+                            self.parent.diagnostics.force_refresh_document(doc, False)
 
             if refresh:
-                await self.parent.diagnostics.refresh()
+                self.parent.diagnostics.refresh()
 
     @language_id("robotframework")
-    @_logger.call
-    async def namespace_invalidated(self, sender: Any, namespace: Namespace) -> None:
-        create_sub_task(self.namespace_invalidated_task(namespace), loop=self.parent.diagnostics.diagnostics_loop)
+    def collect_namespace_diagnostics(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+        return document.get_cache(self._collect_namespace_diagnostics)
 
-    @language_id("robotframework")
-    @threaded()
-    @_logger.call
-    async def collect_namespace_diagnostics(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
-        return await document.get_cache(self._collect_namespace_diagnostics)
-
-    async def _collect_namespace_diagnostics(self, document: TextDocument) -> DiagnosticsResult:
+    def _collect_namespace_diagnostics(self, document: TextDocument) -> DiagnosticsResult:
         try:
-            namespace = await self.parent.documents_cache.get_namespace(document)
+            namespace = self.parent.documents_cache.get_namespace(document)
 
-            return DiagnosticsResult(self.collect_namespace_diagnostics, await namespace.get_diagnostisc())
-        except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
+            return DiagnosticsResult(self.collect_namespace_diagnostics, namespace.get_diagnostics())
+        except (CancelledError, SystemExit, KeyboardInterrupt):
             raise
         except BaseException as e:
             self._logger.exception(e)
@@ -89,10 +122,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                 [
                     Diagnostic(
                         range=Range(
-                            start=Position(
-                                line=0,
-                                character=0,
-                            ),
+                            start=Position(line=0, character=0),
                             end=Position(
                                 line=len(document.get_lines()),
                                 character=len((document.get_lines())[-1] or ""),
@@ -107,7 +137,11 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
             )
 
     def _create_error_from_node(
-        self, node: ast.AST, msg: str, source: Optional[str] = None, only_start: bool = True
+        self,
+        node: ast.AST,
+        msg: str,
+        source: Optional[str] = None,
+        only_start: bool = True,
     ) -> Diagnostic:
         from robot.parsing.model.statements import Statement
 
@@ -137,28 +171,27 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
         )
 
     @language_id("robotframework")
-    @threaded()
     @_logger.call
-    async def collect_token_errors(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
-        return await document.get_cache(self._collect_token_errors)
+    def collect_token_errors(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+        return document.get_cache(self._collect_token_errors)
 
-    async def _collect_token_errors(self, document: TextDocument) -> DiagnosticsResult:
+    def _collect_token_errors(self, document: TextDocument) -> DiagnosticsResult:
         from robot.errors import VariableError
         from robot.parsing.lexer.tokens import Token
 
         result: List[Diagnostic] = []
         try:
-            for token in await self.parent.documents_cache.get_tokens(document):
-                await check_canceled()
+            for token in self.parent.documents_cache.get_tokens(document):
+                check_current_task_canceled()
 
-                if token.type in [Token.ERROR, Token.FATAL_ERROR] and not Namespace.should_ignore(
-                    document, range_from_token(token)
-                ):
+                if token.type in [
+                    Token.ERROR,
+                    Token.FATAL_ERROR,
+                ] and not Namespace.should_ignore(document, range_from_token(token)):
                     result.append(self._create_error_from_token(token))
 
                 try:
                     for variable_token in token.tokenize_variables():
-                        await check_canceled()
                         if variable_token == token:
                             break
 
@@ -179,7 +212,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                                 code=type(e).__qualname__,
                             )
                         )
-        except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
+        except (CancelledError, SystemExit, KeyboardInterrupt):
             raise
         except BaseException as e:
             return DiagnosticsResult(
@@ -187,10 +220,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                 [
                     Diagnostic(
                         range=Range(
-                            start=Position(
-                                line=0,
-                                character=0,
-                            ),
+                            start=Position(line=0, character=0),
                             end=Position(
                                 line=len(document.get_lines()),
                                 character=len((document.get_lines())[-1] or ""),
@@ -207,20 +237,18 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
         return DiagnosticsResult(self.collect_token_errors, result)
 
     @language_id("robotframework")
-    @threaded()
     @_logger.call
-    async def collect_model_errors(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
-        return await document.get_cache(self._collect_model_errors)
-
-    async def _collect_model_errors(self, document: TextDocument) -> DiagnosticsResult:
-        from robotcode.language_server.robotframework.utils.ast_utils import HasError, HasErrors
-        from robotcode.language_server.robotframework.utils.async_ast import iter_nodes
+    def collect_model_errors(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+        return document.get_cache(self._collect_model_errors)
 
+    def _collect_model_errors(self, document: TextDocument) -> DiagnosticsResult:
         try:
-            model = await self.parent.documents_cache.get_model(document, True)
+            model = self.parent.documents_cache.get_model(document, True)
 
             result: List[Diagnostic] = []
-            async for node in iter_nodes(model):
+            for node in iter_nodes(model):
+                check_current_task_canceled()
+
                 error = node.error if isinstance(node, HasError) else None
                 if error is not None and not Namespace.should_ignore(document, range_from_node(node)):
                     result.append(self._create_error_from_node(node, error))
@@ -232,7 +260,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
 
             return DiagnosticsResult(self.collect_model_errors, result)
 
-        except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
+        except (CancelledError, SystemExit, KeyboardInterrupt):
             raise
         except BaseException as e:
             return DiagnosticsResult(
@@ -240,10 +268,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                 [
                     Diagnostic(
                         range=Range(
-                            start=Position(
-                                line=0,
-                                character=0,
-                            ),
+                            start=Position(line=0, character=0),
                             end=Position(
                                 line=len(document.get_lines()),
                                 character=len((document.get_lines())[-1] or ""),
@@ -258,26 +283,27 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
             )
 
     @language_id("robotframework")
-    @threaded()
     @_logger.call
-    async def collect_unused_keyword_references(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
-        if not self.parent.diagnostics.workspace_loaded_event.is_set():
-            return DiagnosticsResult(self.collect_unused_keyword_references, None)
-
-        config = await self.parent.workspace.get_configuration_async(AnalysisConfig, document.uri)
+    def collect_unused_keyword_references(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+        config = self.parent.workspace.get_configuration(AnalysisConfig, document.uri)
 
         if not config.find_unused_references:
             return DiagnosticsResult(self.collect_unused_keyword_references, [])
 
-        return await self._collect_unused_keyword_references(document)
+        if not self._collect_unused_references_event.is_set():
+            return DiagnosticsResult(self.collect_unused_keyword_references, None, True)
 
-    async def _collect_unused_keyword_references(self, document: TextDocument) -> DiagnosticsResult:
+        return self._collect_unused_keyword_references(document)
+
+    def _collect_unused_keyword_references(self, document: TextDocument) -> DiagnosticsResult:
         try:
-            namespace = await self.parent.documents_cache.get_namespace(document)
+            namespace = self.parent.documents_cache.get_namespace(document)
 
             result: List[Diagnostic] = []
-            for kw in (await namespace.get_library_doc()).keywords.values():
-                references = await self.parent.robot_references.find_keyword_references(document, kw, False, True)
+            for kw in (namespace.get_library_doc()).keywords.values():
+                check_current_task_canceled()
+
+                references = self.parent.robot_references.find_keyword_references(document, kw, False, True)
                 if not references and not Namespace.should_ignore(document, kw.name_range):
                     result.append(
                         Diagnostic(
@@ -291,7 +317,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                     )
 
             return DiagnosticsResult(self.collect_unused_keyword_references, result)
-        except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
+        except (CancelledError, SystemExit, KeyboardInterrupt):
             raise
         except BaseException as e:
             return DiagnosticsResult(
@@ -299,10 +325,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                 [
                     Diagnostic(
                         range=Range(
-                            start=Position(
-                                line=0,
-                                character=0,
-                            ),
+                            start=Position(line=0, character=0),
                             end=Position(
                                 line=len(document.get_lines()),
                                 character=len((document.get_lines())[-1] or ""),
@@ -317,27 +340,33 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
             )
 
     @language_id("robotframework")
-    @threaded()
     @_logger.call
-    async def collect_unused_variable_references(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
-        if not self.parent.diagnostics.workspace_loaded_event.is_set():
-            return DiagnosticsResult(self.collect_unused_variable_references, None)
-
-        config = await self.parent.workspace.get_configuration_async(AnalysisConfig, document.uri)
+    def collect_unused_variable_references(self, sender: Any, document: TextDocument) -> DiagnosticsResult:
+        config = self.parent.workspace.get_configuration(AnalysisConfig, document.uri)
 
         if not config.find_unused_references:
             return DiagnosticsResult(self.collect_unused_variable_references, [])
 
-        return await self._collect_unused_variable_references(document)
+        if not self._collect_unused_references_event.is_set():
+            return DiagnosticsResult(self.collect_unused_variable_references, None, True)
 
-    async def _collect_unused_variable_references(self, document: TextDocument) -> DiagnosticsResult:
+        return self._collect_unused_variable_references(document)
+
+    def _collect_unused_variable_references(self, document: TextDocument) -> DiagnosticsResult:
         try:
-            namespace = await self.parent.documents_cache.get_namespace(document)
+            namespace = self.parent.documents_cache.get_namespace(document)
 
             result: List[Diagnostic] = []
 
-            for var in (await namespace.get_variable_references()).keys():
-                references = await self.parent.robot_references.find_variable_references(document, var, False, True)
+            for var in (namespace.get_variable_references()).keys():
+                check_current_task_canceled()
+
+                if isinstance(
+                    var, (LibraryArgumentDefinition, EnvironmentVariableDefinition, CommandLineVariableDefinition)
+                ):
+                    continue
+
+                references = self.parent.robot_references.find_variable_references(document, var, False, True)
                 if not references and not Namespace.should_ignore(document, var.name_range):
                     result.append(
                         Diagnostic(
@@ -352,7 +381,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                     )
 
             return DiagnosticsResult(self.collect_unused_variable_references, result)
-        except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
+        except (CancelledError, SystemExit, KeyboardInterrupt):
             raise
         except BaseException as e:
             return DiagnosticsResult(
@@ -360,10 +389,7 @@ class RobotDiagnosticsProtocolPart(RobotLanguageServerProtocolPart):
                 [
                     Diagnostic(
                         range=Range(
-                            start=Position(
-                                line=0,
-                                character=0,
-                            ),
+                            start=Position(line=0, character=0),
                             end=Position(
                                 line=len(document.get_lines()),
                                 character=len((document.get_lines())[-1] or ""),
